@@ -15,11 +15,9 @@ from openapi_spec_validator import openapi_v2_spec_validator, openapi_v3_spec_va
 from prance.util.resolver import RefResolver
 
 # noinspection PyProtectedMember
-from prance.util.url import ResolutionError
 from rest_framework.schemas.generators import EndpointEnumerator
 
 from openapi_tester.constants import PARAMETER_CAPTURE_REGEX
-from openapi_tester.exceptions import OpenAPISchemaError
 
 
 def handle_recursion_limit(schema: dict) -> Callable:
@@ -34,25 +32,11 @@ def handle_recursion_limit(schema: dict) -> Callable:
         definition = schema
         for key in keys:
             definition = definition[key]
-        return remove_recursive_ref(definition, fragment)
+        return definition
 
     return handler
 
 
-def remove_recursive_ref(schema: dict, fragment: str) -> dict:
-    """
-    Iterates over a dictionary to look for pesky recursive $refs using the fragment identifier.
-    """
-    for key, value in schema.items():
-        if isinstance(value, dict):
-            if "$ref" in value.keys() and fragment in value["$ref"]:
-                # TODO: use this value in the testing - to ignore some parts of the specs
-                schema[key] = {"x-recursive-ref-replaced": True}
-            else:
-                schema[key] = remove_recursive_ref(schema[key], fragment)
-    return schema
-
-
 class BaseSchemaLoader:
     """
     Base class for OpenAPI schema loading classes.
@@ -82,19 +66,16 @@ class BaseSchemaLoader:
         return self.schema  # type: ignore
 
     def de_reference_schema(self, schema: dict) -> dict:
-        try:
-            url = schema["basePath"] if "basePath" in schema else self.base_path
-            recursion_handler = handle_recursion_limit(schema)
-            resolver = RefResolver(
-                schema,
-                recursion_limit_handler=recursion_handler,
-                recursion_limit=10,
-                url=url,
-            )
-            resolver.resolve_references()
-            return resolver.specs
-        except ResolutionError as e:
-            raise OpenAPISchemaError(e.args[0]) from e
+        url = schema["basePath"] if "basePath" in schema else self.base_path
+        recursion_handler = handle_recursion_limit(schema)
+        resolver = RefResolver(
+            schema,
+            recursion_limit_handler=recursion_handler,
+            recursion_limit=10,
+            url=url,
+        )
+        resolver.resolve_references()
+        return resolver.specs
 
     def normalize_schema_paths(self, schema: dict) -> Dict[str, dict]:
         normalized_paths: Dict[str, dict] = {}
@@ -128,7 +109,7 @@ class BaseSchemaLoader:
         path, resolved_path = self.resolve_path(de_parameterized_path)
         for parameter in list(re.findall(PARAMETER_CAPTURE_REGEX, path)):
             parameter_name = parameter.replace("{", "").replace("}", "")
-            path = path.replace(resolved_path.kwargs[parameter_name], parameter_name)
+            path = path.replace(str(resolved_path.kwargs[parameter_name]), parameter_name)
         return path
 
     @staticmethod
@@ -159,13 +140,13 @@ class BaseSchemaLoader:
                 return path, resolved_route
             except Resolver404:
                 continue
-        paths = self.get_endpoint_paths()
-        closest_matches = "".join(f"\n- {i}" for i in difflib.get_close_matches(endpoint_path, paths))
+        message = f"Could not resolve path `{endpoint_path}`."
+        closest_matches = "".join(
+            f"\n- {i}" for i in difflib.get_close_matches(endpoint_path, self.get_endpoint_paths())
+        )
         if closest_matches:
-            raise ValueError(
-                f"Could not resolve path `{endpoint_path}`.\n\nDid you mean one of these?{closest_matches}"
-            )
-        raise ValueError(f"Could not resolve path `{endpoint_path}`")
+            message += f"\n\nDid you mean one of these?{closest_matches}"
+        raise ValueError(message)
 
 
 class DrfYasgSchemaLoader(BaseSchemaLoader):
@@ -187,20 +168,10 @@ class DrfYasgSchemaLoader(BaseSchemaLoader):
         odict_schema = self.schema_generator.get_schema(None, True)
         return loads(dumps(odict_schema.as_odict()))
 
-    def get_path_prefix(self) -> str:
-        """
-        Returns the drf_yasg specified path prefix.
-
-        Drf_yasg `cleans` schema paths by finding recurring path patterns,
-        and cutting them out of the generated openapi schema.
-        For example, `/api/v1/example` might then just become `/example`
-        """
-
-        return self.schema_generator.determine_path_prefix(self.get_endpoint_paths())
-
     def resolve_path(self, endpoint_path: str) -> tuple:
         de_parameterized_path, resolved_path = super().resolve_path(endpoint_path=endpoint_path)
-        path_prefix = self.get_path_prefix()  # typically might be 'api/' or 'api/v1/'
+        # typically might be 'api/' or 'api/v1/'
+        path_prefix = self.schema_generator.determine_path_prefix(self.get_endpoint_paths())
         if path_prefix == "/":
             path_prefix = ""
         return de_parameterized_path[len(path_prefix) :], resolved_path
@@ -221,23 +192,13 @@ class DrfSpectacularSchemaLoader(BaseSchemaLoader):
         """
         Loads generated schema from drf_spectacular and returns it as a dict.
         """
-        return loads(dumps(self.schema_generator.get_schema(None, True)))
-
-    @staticmethod
-    def get_path_prefix() -> str:
-        """
-        Returns the drf_spectacular specified path prefix.
-        """
-        from drf_spectacular.settings import spectacular_settings
-
-        return spectacular_settings.SCHEMA_PATH_PREFIX
+        return loads(dumps(self.schema_generator.get_schema(public=True)))
 
     def resolve_path(self, endpoint_path: str) -> tuple:
+        from drf_spectacular.settings import spectacular_settings
+
         de_parameterized_path, resolved_path = super().resolve_path(endpoint_path=endpoint_path)
-        path_prefix = self.get_path_prefix()  # typically might be 'api/' or 'api/v1/'
-        if path_prefix == "/":
-            path_prefix = ""
-        return de_parameterized_path[len(path_prefix) :], resolved_path
+        return de_parameterized_path[len(spectacular_settings.SCHEMA_PATH_PREFIX) :], resolved_path
 
 
 class StaticSchemaLoader(BaseSchemaLoader):
