@@ -1,26 +1,22 @@
 """ Schema Tester """
 import re
 from functools import reduce
-from typing import Any, Callable, Dict, List, Optional, Union, cast
+from typing import Any, Callable, Dict, List, Optional, Union
 
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
 from django.utils.dateparse import parse_date, parse_datetime
 from rest_framework.response import Response
-from rest_framework.test import APITestCase
 
 from openapi_tester import type_declarations as td
 from openapi_tester.constants import (
-    ANY_OF_ERROR,
-    EXCESS_RESPONSE_KEY_ERROR,
     INIT_ERROR,
     INVALID_PATTERN_ERROR,
-    MISSING_RESPONSE_KEY_ERROR,
-    NONE_ERROR,
-    ONE_OF_ERROR,
     OPENAPI_PYTHON_MAPPING,
     UNDOCUMENTED_SCHEMA_SECTION_ERROR,
+    VALIDATE_ANY_OF_ERROR,
     VALIDATE_ENUM_ERROR,
+    VALIDATE_EXCESS_RESPONSE_KEY_ERROR,
     VALIDATE_FORMAT_ERROR,
     VALIDATE_MAX_ARRAY_LENGTH_ERROR,
     VALIDATE_MAX_LENGTH_ERROR,
@@ -30,11 +26,14 @@ from openapi_tester.constants import (
     VALIDATE_MIN_LENGTH_ERROR,
     VALIDATE_MINIMUM_ERROR,
     VALIDATE_MINIMUM_NUMBER_OF_PROPERTIES_ERROR,
+    VALIDATE_MISSING_RESPONSE_KEY_ERROR,
     VALIDATE_MULTIPLE_OF_ERROR,
+    VALIDATE_NONE_ERROR,
+    VALIDATE_ONE_OF_ERROR,
     VALIDATE_PATTERN_ERROR,
-    VALIDATE_RESPONSE_TYPE_ERROR,
     VALIDATE_TYPE_ERROR,
     VALIDATE_UNIQUE_ITEMS_ERROR,
+    VALIDATE_WRITE_ONLY_RESPONSE_KEY_ERROR,
 )
 from openapi_tester.exceptions import DocumentationError, OpenAPISchemaError, UndocumentedSchemaSectionError
 from openapi_tester.loaders import DrfSpectacularSchemaLoader, DrfYasgSchemaLoader, StaticSchemaLoader
@@ -97,7 +96,7 @@ class SchemaTester:
             UNDOCUMENTED_SCHEMA_SECTION_ERROR.format(key=status_code, error_addon=error_addon)
         )
 
-    def get_response_schema_section(self, response: td.Response) -> dict:
+    def get_response_schema_section(self, response: td.Response) -> Dict[str, Any]:
         """
         Fetches the response section of a schema, wrt. the route, method, status code, and schema version.
 
@@ -105,6 +104,7 @@ class SchemaTester:
         :return dict
         """
         schema = self.loader.get_schema()
+        response_method = response.request["REQUEST_METHOD"].lower()
         parameterized_path = self.loader.parameterize_path(response.request["PATH_INFO"])
         paths_object = self._get_key_value(schema, "paths")
 
@@ -112,12 +112,14 @@ class SchemaTester:
         route_object = self._get_key_value(
             paths_object,
             parameterized_path,
-            f"\n\nFor debugging purposes, other valid routes include: \n\n\t• {pretty_routes}",
+            f"\n\nValid routes include: \n\n\t• {pretty_routes}",
         )
 
         str_methods = ", ".join(method.upper() for method in route_object.keys() if method.upper() != "PARAMETERS")
         method_object = self._get_key_value(
-            route_object, response.request["REQUEST_METHOD"].lower(), f"\n\nAvailable methods include: {str_methods}."
+            route_object,
+            response_method,
+            f"\n\nAvailable methods include: {str_methods}.",
         )
 
         responses_object = self._get_key_value(method_object, "responses")
@@ -131,8 +133,16 @@ class SchemaTester:
         if "openapi" not in schema:
             # openapi 2.0, i.e. "swagger" has a different structure than openapi 3.0 status sub-schemas
             return self._get_key_value(status_code_object, "schema")
-        content_object = self._get_key_value(status_code_object, "content")
-        json_object = self._get_key_value(content_object, "application/json")
+        content_object = self._get_key_value(
+            status_code_object,
+            "content",
+            f"No content documented for method: {response_method}, path: {parameterized_path}",
+        )
+        json_object = self._get_key_value(
+            content_object,
+            "application/json",
+            f"no `application/json` responses documented for method: {response_method}, path: {parameterized_path}",
+        )
         return self._get_key_value(json_object, "schema")
 
     def handle_all_of(
@@ -175,7 +185,7 @@ class SchemaTester:
                 continue
         if matches != 1:
             raise DocumentationError(
-                message=ONE_OF_ERROR.format(matches=matches),
+                message=VALIDATE_ONE_OF_ERROR.format(matches=matches),
                 response=data,
                 schema=schema_section,
                 reference=f"{reference}.oneOf",
@@ -191,7 +201,8 @@ class SchemaTester:
     ):
         any_of: List[Dict[str, Any]] = schema_section.get("anyOf", [])
         combined_sub_schemas = map(
-            lambda index: reduce(lambda x, y: combine_sub_schemas([x, y]), any_of[index:]), range(len(any_of))
+            lambda index: reduce(lambda x, y: combine_sub_schemas([x, y]), any_of[index:]),
+            range(len(any_of)),
         )
 
         for schema in [*any_of, *combined_sub_schemas]:
@@ -207,11 +218,10 @@ class SchemaTester:
             except DocumentationError:
                 continue
         raise DocumentationError(
-            message=ANY_OF_ERROR,
+            message=VALIDATE_ANY_OF_ERROR,
             response=data,
             schema=schema_section,
             reference=f"{reference}.anyOf",
-            hint="",
         )
 
     @staticmethod
@@ -233,7 +243,10 @@ class SchemaTester:
         )
 
     def _validate_key_casing(
-        self, key: str, case_tester: Optional[Callable[[str], None]] = None, ignore_case: Optional[List[str]] = None
+        self,
+        key: str,
+        case_tester: Optional[Callable[[str], None]] = None,
+        ignore_case: Optional[List[str]] = None,
     ) -> None:
         tester = case_tester or getattr(self, "case_tester", None)
         ignore_case = [*self.ignore_case, *(ignore_case or [])]
@@ -263,7 +276,7 @@ class SchemaTester:
         valid = True
         schema_format = schema_section.get("format")
         if schema_format in ["double", "float"]:
-            valid = isinstance(data, float)
+            valid = isinstance(data, float) if data != 0 else isinstance(data, int)
         elif schema_format == "byte":
             valid = isinstance(data, bytes)
         elif schema_format in ["date", "date-time"]:
@@ -292,7 +305,10 @@ class SchemaTester:
         return (
             None
             if valid
-            else VALIDATE_TYPE_ERROR.format(expected=OPENAPI_PYTHON_MAPPING[schema_type], received=type(data).__name__)
+            else VALIDATE_TYPE_ERROR.format(
+                expected=OPENAPI_PYTHON_MAPPING[schema_type],
+                received=type(data).__name__,
+            )
         )
 
     @staticmethod
@@ -362,8 +378,6 @@ class SchemaTester:
             return schema["type"]
         if "properties" in schema or "additionalProperties" in schema:
             return "object"
-        if "items" in schema:
-            return "array"
         return None
 
     def test_schema_section(
@@ -382,7 +396,7 @@ class SchemaTester:
                 # If data is None and nullable, we return early
                 return
             raise DocumentationError(
-                message=NONE_ERROR.format(expected=OPENAPI_PYTHON_MAPPING[schema_section.get("type", "")]),
+                message=VALIDATE_NONE_ERROR.format(expected=OPENAPI_PYTHON_MAPPING[schema_section.get("type", "")]),
                 response=data,
                 schema=schema_section,
                 reference=reference,
@@ -433,7 +447,12 @@ class SchemaTester:
         for validator in validators:
             error = validator(schema_section, data)
             if error:
-                raise DocumentationError(message=error, response=data, schema=schema_section, reference=reference)
+                raise DocumentationError(
+                    message=error,
+                    response=data,
+                    schema=schema_section,
+                    reference=reference,
+                )
 
         if schema_section_type == "object":
             self.test_openapi_object(
@@ -468,7 +487,8 @@ class SchemaTester:
         """
 
         properties = schema_section.get("properties", {})
-        required_keys = schema_section.get("required", [])
+        write_only_properties = [key for key in properties.keys() if properties[key].get("writeOnly")]
+        required_keys = [key for key in schema_section.get("required", []) if key not in write_only_properties]
         response_keys = data.keys()
         additional_properties: Optional[Union[bool, dict]] = schema_section.get("additionalProperties")
         if not properties and isinstance(additional_properties, dict):
@@ -477,7 +497,7 @@ class SchemaTester:
             self._validate_key_casing(key, case_tester, ignore_case)
             if key in required_keys and key not in response_keys:
                 raise DocumentationError(
-                    message=MISSING_RESPONSE_KEY_ERROR.format(missing_key=key),
+                    message=VALIDATE_MISSING_RESPONSE_KEY_ERROR.format(missing_key=key),
                     hint="Remove the key from your OpenAPI docs, or include it in your API response.",
                     response=data,
                     schema=schema_section,
@@ -489,12 +509,20 @@ class SchemaTester:
             additional_properties_allowed = additional_properties is True
             if key not in properties and not key_in_additional_properties and not additional_properties_allowed:
                 raise DocumentationError(
-                    message=EXCESS_RESPONSE_KEY_ERROR.format(excess_key=key),
+                    message=VALIDATE_EXCESS_RESPONSE_KEY_ERROR.format(excess_key=key),
                     hint="Remove the key from your API response, or include it in your OpenAPI docs.",
                     response=data,
                     schema=schema_section,
                     reference=f"{reference}.object:key:{key}",
                 )
+            if key in write_only_properties:
+                raise DocumentationError(
+                    message=VALIDATE_WRITE_ONLY_RESPONSE_KEY_ERROR.format(write_only_key=key),
+                    hint="Remove the key from your API response, or remove the `WriteOnly` restriction.",
+                    response=data,
+                    schema=schema_section,
+                    reference=f"{reference}.object:key:{key}",
+                )
         for key, value in data.items():
             self.test_schema_section(
                 schema_section=properties[key],
@@ -512,19 +540,10 @@ class SchemaTester:
         case_tester: Optional[Callable[[str], None]],
         ignore_case: Optional[List[str]],
     ) -> None:
-        items = schema_section["items"]  # the items keyword is required in arrays
-        if data and not items:
-            raise DocumentationError(
-                message="Mismatched content. Response list contains data when the schema is empty.",
-                response=data,
-                schema=schema_section,
-                reference=f"{reference}.array",
-                hint="Document the contents of the empty dictionary to match the response object.",
-            )
-
         for datum in data:
             self.test_schema_section(
-                schema_section=items,
+                # the items keyword is required in arrays
+                schema_section=schema_section["items"],
                 data=datum,
                 reference=f"{reference}.array.item",
                 case_tester=case_tester,
@@ -533,7 +552,7 @@ class SchemaTester:
 
     def validate_response(
         self,
-        response: td.Response,
+        response: Response,
         case_tester: Optional[Callable[[str], None]] = None,
         ignore_case: Optional[List[str]] = None,
     ):
@@ -546,10 +565,6 @@ class SchemaTester:
         :raises: ``openapi_tester.exceptions.DocumentationError`` for inconsistencies in the API response and schema.
                  ``openapi_tester.exceptions.CaseError`` for case errors.
         """
-
-        if not isinstance(response, Response):
-            raise ValueError(VALIDATE_RESPONSE_TYPE_ERROR)
-
         response_schema = self.get_response_schema_section(response)
         self.test_schema_section(
             schema_section=response_schema,
@@ -558,18 +573,3 @@ class SchemaTester:
             case_tester=case_tester,
             ignore_case=ignore_case,
         )
-
-    def test_case(self) -> APITestCase:
-        validate_response = self.validate_response
-
-        def assert_response(
-            response: td.Response,
-            case_tester: Optional[Callable[[str], None]] = None,
-            ignore_case: Optional[List[str]] = None,
-        ) -> None:
-            """
-            Assert response matches the OpenAPI spec.
-            """
-            validate_response(response=response, case_tester=case_tester, ignore_case=ignore_case)
-
-        return cast(td.OpenAPITestCase, type("OpenAPITestCase", (APITestCase,), {"assertResponse": assert_response}))
